import logging

from whatsapp_business_api_is.utils import format_number


class BaseMsg:
    def __init__(self, msg):
        self.number = msg['from']
        self.type = msg['type']
        self.raw_msg = msg
        self.validated_data = None


class TextMsg(BaseMsg):
    def __init__(self, msg):
        super().__init__(msg)
        self.text = msg['text']['body'].strip().replace(u'\xa0', u' ')  # sometimes a space represent by \xa0


class InteractiveMsg(BaseMsg):
    def __init__(self, msg):
        super().__init__(msg)
        self.button_reply_id = msg['interactive']['button_reply']['id']


class ButtonMsg(BaseMsg):
    def __init__(self, msg):
        super().__init__(msg)
        self.button_text = msg['button']['text']


class ContactsMsg(BaseMsg):
    class Contact:
        def __init__(self, number, name):
            self.number = number
            self.name = name

    def __init__(self, msg):
        super().__init__(msg)
        self.contacts = []
        for contact in msg['contacts']:
            if len(contact['phones']) == 1:
                number = contact['phones'][0]['wa_id'] or format_number(
                    contact['phones'][0]['phone'])  # TODO ignore non whatsapp numbers?
                self.contacts.append(
                    ContactsMsg.Contact(number, contact['name']['formatted_name']))


class ImagesMsg(BaseMsg):
    def __init__(self, msg):
        super().__init__(msg)
        self.image_id = msg['image']['id']


def msg_factory(msg):
    msg_classes = {
        'text': TextMsg,
        'interactive': InteractiveMsg,
        'button': ButtonMsg,
        'contacts': ContactsMsg,
        'image': ImagesMsg
    }
    msg_type = msg['type']
    msg_class = msg_classes.get(msg_type, BaseMsg)
    logging.info(f"Got message with {msg_type=} -> {msg_class=}")

    return msg_class(msg)



from django.db import models
from django.db.models.signals import post_save

from whatsapp_business_api_is.conf import Conf

# Create your models here.
TYPE_USER_START = 'user_start'
TYPE_QUICK_REPLY = 'quick_reply'
TYPE_CALL_TO_ACTION = 'call_to_action'
TYPE_TEXT = 'text'
TYPE_MEDIA = 'media'
TYPE_NUMBER = 'number'
TYPE_CHOICES = 'choices'
TYPE_GENERIC = 'generic'
MESSAGE_TYPES = [
    (TYPE_USER_START, TYPE_USER_START),
    (TYPE_QUICK_REPLY, TYPE_QUICK_REPLY),
    (TYPE_CALL_TO_ACTION, TYPE_CALL_TO_ACTION),
    (TYPE_TEXT, TYPE_TEXT),
    (TYPE_MEDIA, TYPE_MEDIA),
    (TYPE_NUMBER, TYPE_NUMBER),
    (TYPE_CHOICES, TYPE_CHOICES),
    (TYPE_GENERIC, TYPE_GENERIC)
]


class OutgoingMessage(models.Model):
    DEFAULT_STATE = 'initial'

    key = models.CharField(primary_key=True, max_length=100)
    type = models.TextField(choices=MESSAGE_TYPES, default=TYPE_TEXT)
    template_name = models.TextField(default=None, null=True)
    message_variables = models.JSONField(default=None, null=True)
    quick_reply = models.JSONField(default=None, null=True)
    choices = models.JSONField(default=None, null=True)
    text = models.CharField(null=True, db_index=True, max_length=500)
    variable = models.CharField(null=True, max_length=250)
    actions = models.JSONField(default=None, null=True)
    next_message = models.ForeignKey('self', default=None, null=True, on_delete=models.CASCADE)
    skip_if_exists = models.JSONField(default=None, null=True)

    def __unicode__(self):
        return u'{0}'.format(self.key)


class IncomingMessage(models.Model):
    key = models.CharField(primary_key=True, max_length=100)
    type = models.TextField(choices=MESSAGE_TYPES, default=TYPE_USER_START)
    pattern = models.CharField(null=True, db_index=True, max_length=250)
    actions = models.JSONField(default=None, null=True)
    message = models.ForeignKey(OutgoingMessage, null=True, on_delete=models.CASCADE, related_name='responses')
    reply = models.ForeignKey(OutgoingMessage, null=True, on_delete=models.CASCADE)
    force_next = models.BooleanField(default=False)
    is_default = models.BooleanField(
        default=False)  # relevant only for message with multiple responses and skip_if_exists.
    validators = models.JSONField(default=None, null=True)

    def __unicode__(self):
        return u'{0}'.format(self.key)


class WaUser(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    name = models.TextField(blank=True)
    number = models.CharField(primary_key=True, max_length=13)
    email = models.EmailField(null=True)
    state = models.ForeignKey(OutgoingMessage, on_delete=models.CASCADE, default=OutgoingMessage.DEFAULT_STATE)
    opt_in = models.BooleanField(default=False)

    def __unicode__(self):
        return u'{}'.format(self.number)


def outgoing_message_post_save(sender, instance, *args, **kwargs):
    message = instance
    if message.template_name and '%%env%%' in message.template_name:
        message.template_name = message.template_name.replace('%%env%%', Conf.ENV)
        message.save()
        return

    if message.type == TYPE_CHOICES and message.choices:
        for choice, reply_data in message.choices.items():
            reply_key = reply_data.pop('reply_key')
            if 'pattern' not in reply_data:
                reply_data['pattern'] = choice
            IncomingMessage.objects.update_or_create(
                key=f"{message.key}_resp_{choice}",
                defaults={
                    **reply_data,
                    'type': TYPE_CHOICES,
                    'message': message,
                    'reply': OutgoingMessage.objects.get_or_create(key=reply_key)[0]
                }
            )


post_save.connect(outgoing_message_post_save, sender=OutgoingMessage)




import json
import logging

from django.db.transaction import non_atomic_requests
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

from whatsapp_business_api_is.apps import WhatsappBusinessApiIsConfig


@csrf_exempt
@require_POST
@non_atomic_requests
def webhook(request):
    jsondata = request.body
    data = json.loads(jsondata)

    logging.info("Data received from Webhook is: ", data)

    if "messages" not in data:
        return HttpResponse("No messages.", content_type="text/plain")

    parser = WhatsappBusinessApiIsConfig.incoming_parser
    for message in data["messages"]:
        logging.info("message received")
        logging.debug(message)
        parser.delay(message)
        logging.info("task called")

    return HttpResponse("Message received okay.", content_type="text/plain")


from django.conf import settings


class Conf:
    """
    Configuration class
    """

    try:
        conf = settings.WAB_IS
    except AttributeError:
        conf = {}

    DEBUG = conf.get("debug", False)

    ENV = conf.get("env", "production")

    DEMO_MODE = conf.get("demo_mode", False)

    D360_BASE_URL = conf.get("d360_base_url", "https://waba-sandbox.360dialog.io/v1/")

    D360_API_KEY = conf.get("d360_api_key", "")

    AUTH_HEADER = conf.get("auth_header", {'D360-API-KEY': D360_API_KEY, })

    TO_FIELD_NAME = conf.get("to_field_name", "to")

    INCOMING_PARSER = conf.get("incoming_parser", "whatsapp_business_api_is.tasks.parse_incoming_message")

    SET_STATE = conf.get("set_state", None)

    DEFAULT_NUMBER_PREFIX = conf.get("default_number_prefix", "972")

    TEMPLATE_LANG_CODE = conf.get("template_lang_code", "he")



import logging
from inspect import getmembers

from django.apps import AppConfig
from django.conf import settings
from django.utils.module_loading import import_string

from whatsapp_business_api_is.conf import Conf


class WhatsappBusinessApiIsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'whatsapp_business_api_is'
    FUNCTIONS = {}
    VALIDATORS = {}
    incoming_parser = None
    set_state = None

    def ready(self):
        try:
            WhatsappBusinessApiIsConfig.incoming_parser = import_string(Conf.INCOMING_PARSER)
            assert callable(WhatsappBusinessApiIsConfig.incoming_parser)
        except AssertionError:
            raise ValueError(f'Conf.INCOMING_PARSER is not a function. Got: {Conf.INCOMING_PARSER}')

        if Conf.SET_STATE:
            try:
                WhatsappBusinessApiIsConfig.set_state = import_string(Conf.SET_STATE)
                assert callable(WhatsappBusinessApiIsConfig.set_state)
            except AssertionError:
                raise ValueError(f'Conf.SET_STATE is not a function. Got: {Conf.SET_STATE}')

        for app in settings.INSTALLED_APPS:

            try:
                validators = import_string(f"{app}.bot_validators.Validators")

                for name, func in getmembers(validators):
                    if name.startswith('_'):
                        continue
                    self.VALIDATORS[f"{app}.{name}"] = func
            except ImportError:
                pass

            try:
                functions = import_string(f"{app}.bot_functions.Functions")

                for name, func in getmembers(functions):
                    if name.startswith('_'):
                        continue
                    if name in self.FUNCTIONS:
                        raise ValueError(f"duplicate key '{name}' found")
                    self.FUNCTIONS[name] = func
            except ImportError:
                pass

        logging.debug("\n\n[Functions]\n  . " + '\n  . '.join(self.FUNCTIONS.keys()))
        logging.debug("\n\n[validators]\n  . " + '\n  . '.join(self.VALIDATORS.keys()))



# Generated by Django 4.0.3 on 2022-11-05 21:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='OutgoingMessage',
            fields=[
                ('key', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('type', models.TextField(choices=[('user_start', 'user_start'), ('quick_reply', 'quick_reply'), ('call_to_action', 'call_to_action'), ('text', 'text'), ('media', 'media'), ('number', 'number'), ('choices', 'choices'), ('generic', 'generic')], default='text')),
                ('template_name', models.TextField(default=None, null=True)),
                ('message_variables', models.JSONField(default=None, null=True)),
                ('quick_reply', models.JSONField(default=None, null=True)),
                ('choices', models.JSONField(default=None, null=True)),
                ('text', models.CharField(db_index=True, max_length=500, null=True)),
                ('variable', models.CharField(max_length=250, null=True)),
                ('actions', models.JSONField(default=None, null=True)),
                ('skip_if_exists', models.JSONField(default=None, null=True)),
                ('next_message', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='whatsapp_business_api_is.outgoingmessage')),
            ],
        ),
        migrations.CreateModel(
            name='WaUser',
            fields=[
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('name', models.TextField(blank=True)),
                ('number', models.CharField(max_length=13, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=254, null=True)),
                ('opt_in', models.BooleanField(default=False)),
                ('state', models.ForeignKey(default='initial', on_delete=django.db.models.deletion.CASCADE, to='whatsapp_business_api_is.outgoingmessage')),
            ],
        ),
        migrations.CreateModel(
            name='IncomingMessage',
            fields=[
                ('key', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('type', models.TextField(choices=[('user_start', 'user_start'), ('quick_reply', 'quick_reply'), ('call_to_action', 'call_to_action'), ('text', 'text'), ('media', 'media'), ('number', 'number'), ('choices', 'choices'), ('generic', 'generic')], default='user_start')),
                ('pattern', models.CharField(db_index=True, max_length=250, null=True)),
                ('actions', models.JSONField(default=None, null=True)),
                ('force_next', models.BooleanField(default=False)),
                ('is_default', models.BooleanField(default=False)),
                ('validators', models.JSONField(default=None, null=True)),
                ('message', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='responses', to='whatsapp_business_api_is.outgoingmessage')),
                ('reply', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='whatsapp_business_api_is.outgoingmessage')),
            ],
        ),
    ]



import logging
import re
from datetime import timedelta, time

from dateutil.parser import parse
from django.apps import apps
from django.core.exceptions import ObjectDoesNotExist, ValidationError

from whatsapp_business_api_is.apps import WhatsappBusinessApiIsConfig
from whatsapp_business_api_is.conf import Conf
from whatsapp_business_api_is.models import TYPE_USER_START, IncomingMessage, WaUser

UUID_PATTERN = re.compile('id:[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}')
TIME_PATTERN = re.compile('^(?P<hours>1[0-9]|2[0-3]|0?[0-9])\D?(?P<minutes>[1-5][0-9]|0?[0-9])$')
DATE_PATTERN = re.compile('^(3[01]|[12][0-9]|0?[1-9])[/.-](1[0-2]|0?[1-9])[/.-](?:20)?[0-9]{2}$')


def get_user(number):
    try:
        user = WaUser.objects.filter(number=number).first()
        logging.info(f'Got {user=}')
    except ObjectDoesNotExist:
        user = None
    return user


def get_start_message(pattern):
    start_message = IncomingMessage.objects.filter(type=TYPE_USER_START,
                                                     pattern=pattern).first()
    return start_message


def get_date(msg_text):
    if not DATE_PATTERN.match(msg_text):
        raise ValidationError('Wrong date format')
    date = parse(msg_text, dayfirst=True)
    return date.date()


def get_time(msg_text):
    m = TIME_PATTERN.match(msg_text)
    return time(int(m.group('hours')), int(m.group('minutes')))


def validate_value(user, msg, msg_obj: 'IncomingMessage'):
    msg_text = ''
    try:
        match msg_obj.type:
            case 'text':
                msg_text = msg.text
                value = msg_text
            case 'date':
                msg_text = msg.text
                value = get_date(msg_text)
            case 'time':
                msg_text = msg.text
                value = get_time(msg_text)
            case 'number':
                msg_text = msg.text
                value = float(msg_text) if '.' in msg_text else int(msg_text)
            case 'quick_reply':
                value = msg_obj.key  # validation happens before
            case 'choices':  # TODO better validation
                value = msg.text if hasattr(msg, 'text') else msg
            case _:
                value = msg
        msg.validated_value = value
    except Exception as e:
        raise ValidationError(f'{msg_text} is not in the correct format for {msg_obj.type}: {e}')

    if msg_obj.validators:
        for validator in msg_obj.validators:
            logging.info(f"run validation: {validator}")
            WhatsappBusinessApiIsConfig.VALIDATORS[validator['name']](user, msg, msg_obj, validator.get('message', ''))


def format_number(number):
    number = re.sub('\D', '', number)

    if number.startswith('0'):
        number = Conf.DEFAULT_NUMBER_PREFIX + number[1:]
    return number


def reschedule_from_shabbat(date):
    # TODO move to parameters
    # isoweekday() method to get the weekday of a given date as an integer, where Monday is 1 and Sunday is 7
    if date.isoweekday() == 5 and date.time() > time(16, 0):
        date += timedelta(days=1)
    if date.isoweekday() == 6 and date.time() < time(21, 0):
        date = date.replace(hour=21, minute=0)
    return date


def reschedule_from_midnight(date):
    # TODO move to parameters
    if time(23, 00) <= date.time():
        return date.replace(day=date.day + 1, hour=8, minute=0)
    elif date.time() < time(8, 0):
        return date.replace(hour=8, minute=0)
    else:
        return date


def parse_filter(row_filters, user):
    filters = {}
    try:
        for k, v in row_filters.items():
            if isinstance(v, str):
                row_filter = v.split('#')
                match row_filter[0]:
                    case 'current_user':
                        obj = user
                    case _:
                        obj = row_filter[0]
                for key in row_filter[1:]:
                    obj = getattr(obj, key)
                filters[k] = obj
            else:
                filters[k] = v
    except Exception as e:
        logging.error(f" Failed to parse filter: {e}")

    return filters


def _get_object(user, data):
    model = apps.get_model(*data['model'])
    filters = parse_filter(data['filter'], user)
    obj = model.objects.filter(**filters).first()
    logging.debug(f"Object found: {model=} {filters=} {obj=}")
    return obj


def run_action(action, user, msg, wab_bot_message, data):
    if action_func := WhatsappBusinessApiIsConfig.FUNCTIONS.get(action, None):
        res = action_func(user, msg, wab_bot_message, data)
        user.refresh_from_db()
        return res
    else:
        logging.info(f"Action '{action}' not found")
    return None


def run_actions(user, msg, wab_bot_message):
    if not wab_bot_message:
        return
    actions = {wab_bot_message.key: None}

    if wab_bot_message.actions:
        actions.update(wab_bot_message.actions)

    for action, data in actions.items():
        logging.info(f'About to run action: {action}')
        run_action(action, user, msg, wab_bot_message, data)


def get_data(user, data):
    obj = None
    if 'model' in data:
        obj = _get_object(user, data)
    if 'action' in data:
        obj = run_action(data.get('action'), user, None, None, data)
    res = getattr(obj, data['field'], None) if 'field' in data else obj

    logging.info(f"{res=}")

    return res


def set_data(user, data, msg):
    obj = None
    if 'model' in data:
        obj = _get_object(user, data)
    if 'action' in data:
        obj = run_action(data.get('action'), user, msg, None, data)

    if 'field' in data:
        value = data.get('value', msg.validated_value)

        logging.info(f"About to save data: {obj=} {data['field']=} {value=}")
        setattr(obj, data['field'], value)
        obj.save(update_fields=[data['field']])


def get_quick_replies_as_flat_list(quick_reply):
    return [list(reply.items())[0] for reply in quick_reply]



import copy
import json
import logging
import os
import re

import requests

from whatsapp_business_api_is.conf import Conf
from whatsapp_business_api_is.models import OutgoingMessage
from whatsapp_business_api_is.utils import get_data, get_quick_replies_as_flat_list

MESSAGES_URL = Conf.D360_BASE_URL + 'messages'
MEDIA_URL = Conf.D360_BASE_URL + 'media'

HEADERS = {
    **Conf.AUTH_HEADER,
    'Content-Type': "application/json",
}

COMPONENT_NAME_PATTERN = re.compile('(?P<type>\w+)(%(?P<index>\d))?')


def get_template_message_data(to_number, template_name, components, lang_code=None):
    language_code = lang_code or os.environ.get('TEMPLATE_LANG_CODE', 'he')
    message = {
        Conf.TO_FIELD_NAME: to_number,
        "type": "template",
        "template": {
            "language": {
                "policy": "deterministic",
                "code": language_code
            },
            "name": template_name,
        }
    }
    message['template']['components'] = components

    logging.debug(f"get_template_message_data:\n{message=}")
    return message


def get_media_message_data(to_number, media_data):
    message = {
        "recipient_type": "individual",
        Conf.TO_FIELD_NAME: to_number,
        "type": media_data['media_type'],

        media_data['media_type']: media_data['payload']
    }
    logging.debug(f"get_media_message_data:\n{message=}")
    return message


def get_text_message_data(to_number, text):
    message = {
        Conf.TO_FIELD_NAME: to_number,
        "type": "text",
        "text": {
            "body": text
        }
    }
    logging.debug(f"get_text_message_data:\n{message=}")
    return message


def get_interactive_message_data(parts, to_number):
    message = {
        "recipient_type": "individual",
        Conf.TO_FIELD_NAME: to_number,
        "type": "interactive",
        "interactive": parts
    }
    logging.debug(f"get_interactive_message_data:\n{message=}")
    return message


def create_button(id_, title):
    button = {
        "type": "reply",
        "reply": {
            "id": id_,
            "title": title
        }
    }
    return button


def send_message(message):
    logging.debug(f"response: {MESSAGES_URL} \nresponse: {message=} {HEADERS=}")
    res = requests.post(url=MESSAGES_URL,
                        data=json.dumps(message),
                        headers=HEADERS)

    logging.debug(f"{res=}")
    logging.debug(f"{res.text=}")
    try:
        logging.debug(f"{res.json()=}")
    except Exception:
        pass
    if res.status_code != 201:
        logging.error(f"Something went wrong")
        raise Exception(res.json())


def send_template_message(user, wab_bot_message, components=None):
    components = components or copy.deepcopy(
        wab_bot_message.message_variables)  # we can do this because this is a parsed JSON field
    if components:
        for component in components:
            for parameter in component['parameters']:
                if variable := parameter.pop('variable'):
                    parameter[parameter['type']] = str(get_data(user, variable))

    logging.debug(f"{components=}")
    message = get_template_message_data(user.number, wab_bot_message.template_name, components)

    if Conf.DEMO_MODE:
        logging.info(f"{'*' * 20}\n*   {message=}\n{'*' * 20}")
        text = wab_bot_message.pk
        if wab_bot_message.quick_reply:
            quick_replies = get_quick_replies_as_flat_list(wab_bot_message.quick_reply)
            text = f"{text}: [{[r[1] for r in quick_replies]}]"
        message = get_text_message_data(user.number, text)

    if message:
        send_message(message)


def send_media_message(user, wab_bot_message, message_text=None):
    message_text = message_text or wab_bot_message.text
    if message_text and wab_bot_message.message_variables['caption']:
        variables = {k: get_data(user, v) for k, v in
                     wab_bot_message.message_variables['caption'].items()}
        message_text = message_text.format(**variables)

    payload = wab_bot_message.message_variables['media']['payload']  # TODO make dynamic
    if message_text:
        payload['caption'] = message_text

    media_data = {
        'media_type': wab_bot_message.message_variables['media']['type'],
        'payload': payload
    }

    message = get_media_message_data(user.number, media_data)

    assert message
    send_message(message)


def send_interactive_message(user, wab_bot_message, message_text=None):
    message_text = message_text or wab_bot_message.text

    # we can do this because this is a parsed JSON field
    parts = copy.deepcopy(wab_bot_message.message_variables) or {}
    if body := parts.get('body'):
        variables = {k: get_data(user, v) for k, v in body.pop('variables').items()}
        body['text'] = message_text.format(**variables)
    else:
        parts['body'] = {'text': message_text}

    match wab_bot_message.type:  # there are other type that not implemented yet
        case 'quick_reply':
            parts['type'] = 'button'
            buttons = get_quick_replies_as_flat_list(wab_bot_message.quick_reply)
            parts['action'] = {
                "buttons": [create_button(id, name) for id, name in buttons]
            }

    message = get_interactive_message_data(parts, user.number)

    assert message
    send_message(message)


def send_text_message(user, wab_bot_message, message_text=None):
    message_text = message_text or wab_bot_message.text
    if wab_bot_message.message_variables:
        variables = {k: get_data(user, v) for k, v in
                     wab_bot_message.message_variables.items()}
        message_text = message_text.format(**variables)

    message = get_text_message_data(user.number, message_text)

    assert message
    send_message(message)


def send_unknown_message(user):
    unknown_message = OutgoingMessage.objects.get(key='unknown')
    send_text_message(user, unknown_message)


def send_error_message(user, error):
    if error and error.message and hasattr(error, 'params') and \
            error.params and error.params.get('custom_message', False):
        if message := OutgoingMessage.objects.filter(key=error.message).first():
            send_text_message(user, message)
        else:
            message = get_text_message_data(user.number, error.message)
            send_message(message)
    else:
        unknown_message = OutgoingMessage.objects.get(key='wrong_format')
        send_text_message(user, unknown_message)


def get_media(media_id):
    res = requests.get(
        url=MEDIA_URL + '/' + media_id,
        headers=Conf.AUTH_HEADER
    )
    logging.info(res.__dict__)
    return res




import os
import urllib.parse

from django.core.management.base import BaseCommand

REGISTER_URL = 'https://waba.360dialog.io/v1/configs/webhook'
SANDBOX_REGISTER_URL = 'https://waba-sandbox.360dialog.io/v1/configs/webhook'


class Command(BaseCommand):
    help = 'register API KEY to 360 Dialog'

    def add_arguments(self, parser):
        # Named (optional) arguments
        parser.add_argument(
            '--server_url',
            help='Server url',
        )
        parser.add_argument(
            '--sandbox',
            action='store_true',
            help="Use 360dialog sandbox"
        )

    def handle(self, *args, **options):
        import requests
        import json

        server_url = options['server_url'] or os.environ.get('SERVER_URL')
        webhook_url = urllib.parse.urljoin(server_url, "/wab-is/webhook")

        payload = json.dumps({
            "url": webhook_url
        })
        headers = {
            'D360-API-KEY': os.environ.get('D360_API_KEY'),
            'Content-Type': 'application/json'
        }

        print(f"{payload=}")
        print(f"{headers=}")

        register_url = REGISTER_URL if not options['sandbox'] else SANDBOX_REGISTER_URL
        response = requests.request("POST", register_url, headers=headers, data=payload)

        print(response.text)
